@@ -24,23 +24,25 @@ import tensorflow as tf
 
 from .models import generate_sgns_batch, sgns_likelihood
 from .models import generate_cbow_batch, cbow_likelihood
-def evaluate_on_holdout_set(embedding, data, model="sgns", ws=5, ns=5, batch_size=25000):
+def evaluate_on_holdout_set(embedding, data, model="sgns", ws=5, ns=5, batch_size=25000, reduce_mean=True):
     if not isinstance(data, tf.Tensor):
         data = tf.constant(data)
     valid_batches = len(data) // batch_size
 
-    valid_ll = 0.0
+    valid_ll = tf.constant([], dtype=tf.float64)
     for batch in progressbar.progressbar(range(valid_batches)):
         start_ix = batch_size * batch
         if model == "sgns":
             i,j,x  = generate_sgns_batch(data, ws=ws, ns=ns, batch=batch_size, start_ix=start_ix)
-            valid_ll += tf.reduce_sum(sgns_likelihood(embedding, i, j, x=x))
+            valid_ll = tf.concat([valid_ll, sgns_likelihood(embedding, i, j, x=x)], axis=0)
         elif model == "cbow":
             i,j,x  = generate_cbow_batch(data, ws=ws, ns=ns, batch=batch_size, start_ix=start_ix)
-            valid_ll += tf.reduce_sum(cbow_likelihood(embedding, i, j, x=x))
+            valid_ll = tf.concat([valid_ll, cbow_likelihood(embedding, i, j, x=x)], axis=0)
 
-    valid_ll = valid_ll / (batch_size * valid_batches)
-    return valid_ll
+    if reduce_mean:
+        return tf.reduce_mean(valid_ll)
+    else:
+        return valid_ll
 
 ###################
 # WORD SIMILARITY #
@@ -102,6 +104,57 @@ def evaluate_word_similarity(embedding, dataset_names=None):
 
     return pd.DataFrame(rows, columns=["Dataset", "Rank Correlation", "No. of Observations", "p-value"])
 
+def evaluate_analogy(embedding, dataset, K=25):
+    """
+    Evaluate embedding performance on word similarity tasks.
+    
+    Args:
+        embedding: embedding as pwe.embeddings.Embedding
+        dataset (pd.DataFrame): Dataframe where each row has four words, word1 - word2 + word3 ≈ word4
+    """
+    df = dataset
+    e = embedding
+    columns = df.columns[:4]
+    def words_in_e(row):
+        w1, w2, w3, w4 = row[columns]
+        # force cast to str
+        w1, w2, w3, w4 = str(w1), str(w2), str(w3), str(w4)
+
+        return w1 in e and w2 in e and w3 in e and w4 in e 
+
+    df["included"] = df.apply(words_in_e, axis=1)
+    df = df[df["included"]]
+    df = df[columns]
+    r = len(df)
+    target_words = list(df[columns[-1]])
+
+    X1 = embedding[df[columns[0]]]
+    X2 = embedding[df[columns[1]]]
+    X3 = embedding[df[columns[2]]]
+    X = X1 - X2 + X3
+
+    inv_vocab = {v: k for k, v in e.vocabulary.items()}
+    eliminate = list(range(len(inv_vocab)))
+    eliminate = [0.0 if "_c" in inv_vocab[i] else 1.0 for i in eliminate]
+    eliminate = tf.transpose(tf.constant([eliminate], dtype=tf.float64))
+
+    X, _ = tf.linalg.normalize(X, axis=1)
+    Ex = tf.linalg.tensordot(e.theta, X, axes=[1,1])
+    tiled_eliminate = tf.tile(eliminate, [1,r])
+    Ex = tf.multiply(Ex, tiled_eliminate)
+
+    rows = []
+
+    for i in range(r):
+        y_hat = Ex[:,i]
+        _, tops = tf.math.top_k(y_hat, k=K)
+        topwords = [inv_vocab[int(i)] for i in tops]
+        correct = target_words[i] in topwords
+        rows.append(topwords + [correct])
+    
+    df = pd.DataFrame(rows)
+    return df
+
 
 
 ###############################
